﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Migrations;
using System.Data.Entity.Migrations.Model;
using System.Data.Entity.Migrations.Sql;
using MySql.Data.Entity;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            Database.SetInitializer(new MigrateDatabaseToLatestVersion<TestContext, Configuration>());

            using (var db = new TestContext())
            {
                db.Database.Initialize(false);
            }
        }
    }

    public class TestContext : DbContext
    {
        public TestContext() : base("test") { }

        public DbSet<TestParent> Parents { get; set; }
        public DbSet<TestChild> Children { get; set; }
    }

    public class TestParent
    {
        public Guid Id { get; set; }
        public string Name { get; set; }
    }

    public class TestChild
    {
        public Guid Id { get; set; }
        public string Name { get; set; }

        // remove this property to force a "DROP FOREIGN KEY"
        public virtual TestParent Parent { get; set; }
    }

    internal sealed class Configuration : DbMigrationsConfiguration<TestContext>
    {
        public Configuration()
        {
            AutomaticMigrationsEnabled = true;
            AutomaticMigrationDataLossAllowed = true;

            SetSqlGenerator("MySql.Data.MySqlClient", new SqlGenerator());
            CodeGenerator = new MySqlMigrationCodeGenerator();
        }
    }

    internal class SqlGenerator : MySqlMigrationSqlGenerator
    {
        public override IEnumerable<MigrationStatement> Generate(IEnumerable<MigrationOperation> migrationOperations, string providerManifestToken)
        {
            IEnumerable<MigrationStatement> res = base.Generate(migrationOperations, providerManifestToken);
            foreach (MigrationStatement ms in res)
            {
                var sql = ms.Sql;
                System.Diagnostics.Debug.WriteLine(sql);
                ms.Sql = ms.Sql.Replace("dbo.", "");
                System.Diagnostics.Debug.WriteLine(ms.Sql);
                System.Diagnostics.Debug.Assert(sql == ms.Sql);
            }
            return res;
        }
    }
}
